#========================================================================================
# package      DE_EPAGES::Event::t::unit::ProductEventService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Event::API::ProductEventService
#========================================================================================
package DE_EPAGES::Event::t::unit::ProductEventService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use DE_EPAGES::Event::API::Constants qw (
    EVENT_PRODUCT_CREATION
    EVENT_PRODUCT_DELETION
    EVENT_PRODUCT_UPDATE
);

use strict;



sub test {
    my $self = shift;

    $self->test_createProductCreationEvent();
    $self->test_createProductDeletionEvent();
    $self->test_createProductUpdateEvent();

    $self->test_EventProcessingDisabled();

    done_testing();
}



sub test_createProductCreationEvent {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mocks = $self->_createMocks();
    my $ProductEventService = $Mocks->get('SUT');

    my $Product = $Mocks->get('Product');
    $ProductEventService->createProductCreationEvent($Product);

    my $aLogCommunicationArgs = GetParametersFromMockifyCall($Mocks->get('Logger'), 'logCommunication');
    is($aLogCommunicationArgs->[0], 'DE_EPAGES::Event::API::EventService::_sendEvent', "$SubTestName - test if log was called in right method");
    is_deeply($aLogCommunicationArgs->[1], $self->_createExpectedLogCommunicationData(EVENT_PRODUCT_CREATION), "$SubTestName - test if log was called with right data");

    my $aEncodeRequestArgs = GetParametersFromMockifyCall($Mocks->get('RestClient'), 'encodeRequest');
    is_deeply($aEncodeRequestArgs->[0], $self->_createRequestData(EVENT_PRODUCT_CREATION), "$SubTestName - test if encodeRequest was called with right data");

    my $aPostArgs = GetParametersFromMockifyCall($Mocks->get('HTTPClient'), 'post');
    is($aPostArgs->[0], 'http://event.queue.url', "$SubTestName - test if request is send to right url");
    is_deeply($aPostArgs->[1], {}, "$SubTestName - test if request is send with right headers");
    is($aPostArgs->[2], 'request-data', "$SubTestName - test if request is send with right body");
}



sub test_createProductDeletionEvent {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mocks = $self->_createMocks();
    my $ProductEventService = $Mocks->get('SUT');

    my $Product = $Mocks->get('Product');
    $ProductEventService->createProductDeletionEvent($Product);

    my $aLogCommunicationArgs = GetParametersFromMockifyCall($Mocks->get('Logger'), 'logCommunication');
    is($aLogCommunicationArgs->[0], 'DE_EPAGES::Event::API::EventService::_sendEvent', "$SubTestName - test if log was called in right method");
    is_deeply($aLogCommunicationArgs->[1], $self->_createExpectedLogCommunicationData(EVENT_PRODUCT_DELETION), "$SubTestName - test if log was called with right data");

    my $aEncodeRequestArgs = GetParametersFromMockifyCall($Mocks->get('RestClient'), 'encodeRequest');
    is_deeply($aEncodeRequestArgs->[0], $self->_createRequestData(EVENT_PRODUCT_DELETION), "$SubTestName - test if encodeRequest was called with right data");

    my $aPostArgs = GetParametersFromMockifyCall($Mocks->get('HTTPClient'), 'post');
    is($aPostArgs->[0], 'http://event.queue.url', "$SubTestName - test if request is send to right url");
    is_deeply($aPostArgs->[1], {}, "$SubTestName - test if request is send with right headers");
    is($aPostArgs->[2], 'request-data', "$SubTestName - test if request is send with right body");
}



sub test_createProductUpdateEvent {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mocks = $self->_createMocks();
    my $ProductEventService = $Mocks->get('SUT');

    my $Product = $Mocks->get('Product');
    $ProductEventService->createProductUpdateEvent($Product);

    my $aLogCommunicationArgs = GetParametersFromMockifyCall($Mocks->get('Logger'), 'logCommunication');
    is($aLogCommunicationArgs->[0], 'DE_EPAGES::Event::API::EventService::_sendEvent', "$SubTestName - test if log was called in right method");
    is_deeply($aLogCommunicationArgs->[1], $self->_createExpectedLogCommunicationData(EVENT_PRODUCT_UPDATE), "$SubTestName - test if log was called with right data");

    my $aEncodeRequestArgs = GetParametersFromMockifyCall($Mocks->get('RestClient'), 'encodeRequest');
    is_deeply($aEncodeRequestArgs->[0], $self->_createRequestData(EVENT_PRODUCT_UPDATE), "$SubTestName - test if encodeRequest was called with right data");

    my $aPostArgs = GetParametersFromMockifyCall($Mocks->get('HTTPClient'), 'post');
    is($aPostArgs->[0], 'http://event.queue.url', "$SubTestName - test if request is send to right url");
    is_deeply($aPostArgs->[1], {}, "$SubTestName - test if request is send with right headers");
    is($aPostArgs->[2], 'request-data', "$SubTestName - test if request is send with right body");
}



sub test_EventProcessingDisabled {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mocks = $self->_createMocks();
    my $ProductEventService = $Mocks->get('SUT');

    my $System = $Mocks->get('System');
    $System->set({'EventProcessingEnabled' => 0});

    my $Product = $Mocks->get('Product');
    $ProductEventService->createProductUpdateEvent($Product);

    ok(!WasCalled($Mocks->get('Logger'), 'logCommunication'),"$SubTestName - test if logCommunication was not called");
    ok(!WasCalled($Mocks->get('RestClient'), 'encodeRequest'),"$SubTestName - test if encodeRequest was not called");
    ok(!WasCalled($Mocks->get('HTTPClient'), 'post'),"$SubTestName - test if request was not sent");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['RestClient', 'System', 'Logger'], sub {
        my ($RestClient, $System, $Logger) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Event::API::ProductEventService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createRestClient', $RestClient, ['object', 'hashref']);
        $Mockify->addMockWithReturnValue('_LoadRootObject', $System);
        $Mockify->addMockWithReturnValue('_createLogger', $Logger);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Product', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Site' => $Shop,
            'GUID' => '123-456',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'GUID' => '314-159',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('RestClient', ['HTTPClient'], sub {
        my ($HTTPClient) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::EPagesJ::API::Object::RestClient', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('encodeRequest', 'request-data', ['hashref']);
        $Mockify->addMockWithReturnValue('client', $HTTPClient);
        return $Mockify->getMockObject();
    });

    $Mockery->register('HTTPClient', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::EPagesJ::API::Object::HTTPClient', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('post', undef, ['string', 'hashref', 'string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('System', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Site', {
            'EventQueueUrl' => 'http://event.queue.url',
            'EventProcessingEnabled' => 1,
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('Logger', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Event::API::Logger', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('logCommunication', undef, ['string', 'hashref']);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createRequestData {
    my $self = shift;
    my ($EventType) = @_;
    return {
        'shopGuid' => '314-159',
        'objectGuid' => '123-456',
        'eventType' => $EventType,
    };
}



sub _createExpectedLogCommunicationData {
    my $self = shift;
    my ($EventType) = @_;
    return {
        'Method' => 'POST',
        'URL' => 'http://event.queue.url',
        'Content' => $self->_createRequestData($EventType),
    };
}

__PACKAGE__->runScript();